@@ -1,6 +1,6 @@
 use {
     crate::{Chunk, Op},
-    koto_parser::{ConstantIndex, MetaId},
+    koto_parser::{ConstantIndex, MetaKeyId},
     std::{convert::TryInto, fmt, sync::Arc},
 };
 
@@ -89,13 +89,13 @@ pub enum Instruction {
         register: u8,
         constant: ConstantIndex,
     },
-    LoadExport {
+    LoadNonLocal {
         register: u8,
         constant: ConstantIndex,
     },
-    SetExport {
-        export: ConstantIndex,
-        source: u8,
+    ValueExport {
+        name: ConstantIndex,
+        value: u8,
     },
     Import {
         register: u8,
@@ -337,7 +337,22 @@ pub enum Instruction {
     MetaInsert {
         register: u8,
         value: u8,
-        id: MetaId,
+        id: MetaKeyId,
+    },
+    MetaInsertNamed {
+        register: u8,
+        value: u8,
+        id: MetaKeyId,
+        name: ConstantIndex,
+    },
+    MetaExport {
+        value: u8,
+        id: MetaKeyId,
+    },
+    MetaExportNamed {
+        value: u8,
+        id: MetaKeyId,
+        name: ConstantIndex,
     },
     Access {
         register: u8,
@@ -375,8 +390,8 @@ impl fmt::Display for Instruction {
             LoadFloat { .. } => write!(f, "LoadFloat"),
             LoadInt { .. } => write!(f, "LoadInt"),
             LoadString { .. } => write!(f, "LoadString"),
-            LoadExport { .. } => write!(f, "LoadExport"),
-            SetExport { .. } => write!(f, "SetExport"),
+            LoadNonLocal { .. } => write!(f, "LoadNonLocal"),
+            ValueExport { .. } => write!(f, "ValueExport"),
             Import { .. } => write!(f, "Import"),
             MakeTuple { .. } => write!(f, "MakeTuple"),
             MakeTempTuple { .. } => write!(f, "MakeTempTuple"),
@@ -429,6 +444,9 @@ impl fmt::Display for Instruction {
             Index { .. } => write!(f, "Index"),
             MapInsert { .. } => write!(f, "MapInsert"),
             MetaInsert { .. } => write!(f, "MetaInsert"),
+            MetaInsertNamed { .. } => write!(f, "MetaInsertNamed"),
+            MetaExport { .. } => write!(f, "MetaExport"),
+            MetaExportNamed { .. } => write!(f, "MetaExportNamed"),
             Access { .. } => write!(f, "Access"),
             TryStart { .. } => write!(f, "TryStart"),
             TryEnd => write!(f, "TryEnd"),
@@ -463,13 +481,13 @@ impl fmt::Debug for Instruction {
                 "LoadString\tresult: {}\tconstant: {}",
                 register, constant
             ),
-            LoadExport { register, constant } => write!(
+            LoadNonLocal { register, constant } => write!(
                 f,
-                "LoadExport\tresult: {}\tconstant: {}",
+                "LoadNonLocal\tresult: {}\tconstant: {}",
                 register, constant
             ),
-            SetExport { export, source } => {
-                write!(f, "SetExport\texport: {}\tsource: {}", export, source)
+            ValueExport { name, value } => {
+                write!(f, "ValueExport\tname: {}\tvalue: {}", name, value)
             }
             Import { register, constant } => {
                 write!(f, "Import\t\tresult: {}\tconstant: {}", register, constant)
@@ -786,6 +804,22 @@ impl fmt::Debug for Instruction {
                 "MetaInsert\tmap: {}\t\tvalue: {}\tid: {:?}",
                 register, value, id
             ),
+            MetaInsertNamed {
+                register,
+                value,
+                id,
+                name,
+            } => write!(
+                f,
+                "MetaInsertNamed\tmap: {}\t\tvalue: {}\tid: {:?}\tname: {}",
+                register, value, id, name
+            ),
+            MetaExport { value, id } => write!(f, "MetaExport\tvalue: {}\tid: {:?}", value, id),
+            MetaExportNamed { value, id, name } => write!(
+                f,
+                "MetaExportNamed\tvalue: {}\tid: {:?}\tname: {}",
+                value, id, name
+            ),
             Access { register, map, key } => write!(
                 f,
                 "Access\t\tresult: {}\tmap: {}\t\tkey: {}",
@@ -940,21 +974,21 @@ impl Iterator for InstructionReader {
                 register: get_byte!(),
                 constant: get_u32!() as ConstantIndex,
             }),
-            Op::LoadExport => Some(LoadExport {
+            Op::LoadNonLocal => Some(LoadNonLocal {
                 register: get_byte!(),
                 constant: get_byte!() as ConstantIndex,
             }),
-            Op::LoadExportLong => Some(LoadExport {
+            Op::LoadNonLocalLong => Some(LoadNonLocal {
                 register: get_byte!(),
                 constant: get_u32!() as ConstantIndex,
             }),
-            Op::SetExport => Some(SetExport {
-                export: get_byte!() as ConstantIndex,
-                source: get_byte!(),
+            Op::ValueExport => Some(ValueExport {
+                name: get_byte!() as ConstantIndex,
+                value: get_byte!(),
             }),
-            Op::SetExportLong => Some(SetExport {
-                export: get_u32!() as ConstantIndex,
-                source: get_byte!(),
+            Op::ValueExportLong => Some(ValueExport {
+                name: get_u32!() as ConstantIndex,
+                value: get_byte!(),
             }),
             Op::Import => Some(Import {
                 register: get_byte!(),
@@ -1242,6 +1276,92 @@ impl Iterator for InstructionReader {
                     })
                 }
             }
+            Op::MetaInsertNamed => {
+                let register = get_byte!();
+                let value = get_byte!();
+                let meta_id = get_byte!();
+                let name = get_byte!() as ConstantIndex;
+                if let Ok(id) = meta_id.try_into() {
+                    Some(MetaInsertNamed {
+                        register,
+                        value,
+                        id,
+                        name,
+                    })
+                } else {
+                    Some(Error {
+                        message: format!(
+                            "Unexpected meta id {} found at instruction {}",
+                            meta_id, op_ip
+                        ),
+                    })
+                }
+            }
+            Op::MetaInsertNamedLong => {
+                let register = get_byte!();
+                let value = get_byte!();
+                let meta_id = get_byte!();
+                let name = get_u32!() as ConstantIndex;
+                if let Ok(id) = meta_id.try_into() {
+                    Some(MetaInsertNamed {
+                        register,
+                        value,
+                        id,
+                        name,
+                    })
+                } else {
+                    Some(Error {
+                        message: format!(
+                            "Unexpected meta id {} found at instruction {}",
+                            meta_id, op_ip
+                        ),
+                    })
+                }
+            }
+            Op::MetaExport => {
+                let meta_id = get_byte!();
+                let value = get_byte!();
+                if let Ok(id) = meta_id.try_into() {
+                    Some(MetaExport { id, value })
+                } else {
+                    Some(Error {
+                        message: format!(
+                            "Unexpected meta id {} found at instruction {}",
+                            meta_id, op_ip
+                        ),
+                    })
+                }
+            }
+            Op::MetaExportNamed => {
+                let meta_id = get_byte!();
+                let value = get_byte!();
+                let name = get_byte!() as ConstantIndex;
+                if let Ok(id) = meta_id.try_into() {
+                    Some(MetaExportNamed { id, value, name })
+                } else {
+                    Some(Error {
+                        message: format!(
+                            "Unexpected meta id {} found at instruction {}",
+                            meta_id, op_ip
+                        ),
+                    })
+                }
+            }
+            Op::MetaExportNamedLong => {
+                let meta_id = get_byte!();
+                let value = get_byte!();
+                let name = get_u32!() as ConstantIndex;
+                if let Ok(id) = meta_id.try_into() {
+                    Some(MetaExportNamed { id, value, name })
+                } else {
+                    Some(Error {
+                        message: format!(
+                            "Unexpected meta id {} found at instruction {}",
+                            meta_id, op_ip
+                        ),
+                    })
+                }
+            }
             Op::Access => Some(Access {
                 register: get_byte!(),
                 map: get_byte!(),
